import logging

import botocore

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def get_regions(self):
    logger.info(" ---Inside utils :: get_regions()--- ")
    """Summary

    Returns:
        TYPE: Description
    """

    client = self.session.client('ec2', region_name='us-east-1')
    region_response = {}
    try:
        region_response = client.describe_regions()
    except botocore.exceptions.ClientError as error:
        if error.response['Error']['Code'] == 'AuthFailure':
            logger.error(f" AccessKey credentails not found here: {error}")
            exit(1)
    except botocore.exceptions.NoCredentialsError as e:
        logger.error(f" Unable to locate credentials: {e} ")
        exit(1)

    regions = [region['RegionName'] for region in region_response['Regions']]
    return regions


# returns the list of log groups
def list_log_groups(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_log_groups()--- ")

    log_groups = {}

    for region in regions:
        # Connect to the cloudwatch log service
        client = self.session.client('logs', region_name=region)
        marker = ''
        while True:
            # list all log groups
            if marker == '':
                response = client.describe_log_groups()
            else:
                response = client.describe_log_groups(
                    nextToken=marker
                )
            # adding the list of log groups to the log_groups
            log_groups.setdefault(region, []).extend(response['logGroups'])

            # trying to fetch the nextToken
            try:
                marker = response['nextToken']
                if marker == '':
                    break
            except KeyError:
                break

    return log_groups


# returns the list of rds instances
def list_rds_instances(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_rds_instances()--- ")
    rds_list = {}

    for region in regions:
        client = self.session.client('rds', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_db_instances()
            else:
                response = client.describe_db_instances(
                    Marker=marker
                )
            rds_list.setdefault(region, []).extend(response['DBInstances'])

            try:
                marker = response['Marker']
                if marker == '':
                    break
            except KeyError:
                break

    return rds_list


# return the list of rds clusters
def list_rds_clusters(self, regions: list) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_rds_clusters()--- ")

    cluster_lst = {}

    for region in regions:
        client = self.session.client('rds', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_db_clusters()
            else:
                response = client.describe_db_clusters(
                    Marker=marker
                )
            cluster_lst.setdefault(region, []).extend(response['DBClusters'])

            try:
                marker = response['Marker']
                if marker == '':
                    break
            except KeyError:
                break

    return cluster_lst



"""
Contains the methods to check compliance for cloudwatch
"""

import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks the compliance for cloudwatch-log-group-encrypted
def log_group_encrypted(self, **kwargs) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside cloudwatch :: log_group_encrypted()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id3.18'
    compliance_type = 'Cloudwatch log group encrypted'
    description = 'Checks if a log group in Amazon CloudWatch Logs is encrypted with an AWS Key Management Service (' \
                  'KMS) key'
    resource_type = 'Cloudwatch logs'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    # iterating through all the log groups
    for region, log_groups in kwargs['log_groups'].items():
        for log_group in log_groups:
            # Checking if log_group is encrypted with kms key or not
            try:
                key_id = log_group['kmsKeyId']
                if key_id == '':
                    result = False
                    offenders.append(log_group['logGroupName'])
                    failReason = 'Log group is not encrypted with KMS'
            except KeyError:
                result = False
                offenders.append(log_group['logGroupName'])
                failReason = 'Log group is not encrypted with KMS'

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# compliance check for cloudwatch log group retention period
def log_group_retention_period_check(self, **kwargs) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside cloudwatch :: log_group_retention_period_check()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id3.22'
    compliance_type = 'Cloudwatch log group retention period check'
    description = 'Checks if Amazon CloudWatch LogGroup retention period is set to specific number of days (<=30)'
    resource_type = 'Cloudwatch logs'
    risk_level = 'Low'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    min_retention_time = 30

    # iterating through all the log groups
    for region, log_groups in kwargs['log_groups'].items():
        for log_group in log_groups:
            # checking the retention period of log group
            try:
                retention_period = log_group['retentionInDays']
                if retention_period > min_retention_time:
                    result = False
                    offenders.append(log_group['logGroupName'])
                    failReason = 'Log group has retention period set to either 0 or >30 days'
            except KeyError:
                result = False
                offenders.append(log_group['logGroupName'])
                failReason = 'Log group has retention period set to either 0 or >30 days'

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }



from boto3 import session
import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


__author__ = 'Klera DevOps'
__version__ = '0.0.2'


class aws_client:
    def __init__(self, **kwargs):
        if 'aws_access_key_id' in kwargs.keys() and 'aws_secret_access_key' in kwargs.keys():
            self.session = session.Session(
                aws_access_key_id=kwargs['aws_access_key_id'],
                aws_secret_access_key=kwargs['aws_secret_access_key'],
            )
        elif 'profile_name' in kwargs.keys():
            self.session = session.Session(profile_name=kwargs['profile_name'])

    from .utils import get_regions, list_log_groups, list_rds_instances, list_rds_clusters
    from .cloudwatch import log_group_encrypted, log_group_retention_period_check
    from .rds import rds_logging_enabled, rds_cluster_deletion_protection_enabled, rds_cluster_multi_az_enabled, rds_instance_iam_authentication_enabled

    def get_compliance(self) -> list:
        """
        :return:
        """
        regions = self.get_regions()

        compliance = []

        try:
            log_groups = self.list_log_groups(regions=regions)
        except ClientError as e:
            logger.error("Access Denied")
            compliance.append(self.log_group_encrypted(exception=True, exception_text=e.response['Error']['Code'])),
            compliance.append(self.log_group_retention_period_check(exception=True, exception_text=e.response['Error']['Code']))
        else:
            compliance.append(self.log_group_encrypted(log_groups=log_groups)),
            compliance.append(self.log_group_retention_period_check(log_groups=log_groups))

        try:
            rds_list = self.list_rds_instances(regions=regions)
            # print('rds instances')
            # print(rds_list)
        except ClientError as e:
            compliance.append(self.rds_instance_iam_authentication_enabled(exception=True, exception_text=e.response['Error']['Code']))
            compliance.append(self.rds_logging_enabled(exception=True, exception_text=e.response['Error']['Code']))
        else:
            compliance.append(self.rds_instance_iam_authentication_enabled(rds_lst=rds_list))
            compliance.append(self.rds_logging_enabled(rds_lst=rds_list))

        try:
            rds_clusters = self.list_rds_clusters(regions=regions)
            # print('rds clusters')
            # print(rds_clusters)
        except ClientError as e:
            compliance.append(self.rds_cluster_deletion_protection_enabled(
                exception=True, exception_text=e.response['Error']['Code'])
            )
            compliance.append(self.rds_cluster_multi_az_enabled(
                exception=True, exception_text=e.response['Error']['Code'])
            )
        else:
            compliance.append(self.rds_cluster_deletion_protection_enabled(cluster_list=rds_clusters))
            compliance.append(self.rds_cluster_multi_az_enabled(cluster_list=rds_clusters))

        return compliance



"""
Contains the methods to check compliance for rds
"""

import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks compliance for rds logging enabled
def rds_logging_enabled(self, **kwargs) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside rds :: rds_logging_enabled()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id3.77'
    compliance_type = 'RDS Logging Enabled'
    description = 'Checks that respective logs of Amazon Relational Database Service (Amazon RDS) are enabled'
    resource_type = 'RDS'
    risk_level = 'Low'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, instances in kwargs['rds_lst'].items():
        for instance in instances:
            try:
                if len(instance['EnabledCloudwatchLogsExports']) <= 0:
                    result = False
                    failReason = "RDS logging is not enabled"
                    offenders.append(instance['DBInstanceIdentifier'])
            except KeyError:
                result = False
                failReason = "RDS logging is not enabled"
                offenders.append(instance['DBInstanceIdentifier'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# checks compliance for rds cluster deletion protection enabled
def rds_cluster_deletion_protection_enabled(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside rds :: rds_cluster_deletion_protection_enabled()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id5.13'
    compliance_type = 'Rds Cluster Deletion Protection Enabled'
    description = 'Checks if an Amazon Relational Database Service (Amazon RDS) cluster has deletion protection enabled'
    resource_type = 'RDS'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, clusters in kwargs['cluster_list'].items():
        for cluster in clusters:
            if not cluster['DeletionProtection']:
                result = False
                failReason = "Deletion protection is not enabled"
                offenders.append(cluster['DBClusterIdentifier'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# checks compliance for rds cluster multi az enabled
def rds_cluster_multi_az_enabled(self, **kwargs) -> dict:
    """
    :param self:
    :param kwargs:
    :return:
    """
    logger.info(" ---Inside rds :: rds_cluster_multi_az_enabled()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id5.14'
    compliance_type = 'Rds Cluster Multi Az Enabled'
    description = 'Checks if an Amazon Relational Database Service (Amazon RDS) cluster has Multi-AZ replication ' \
                  'enabled'
    resource_type = 'RDS'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, clusters in kwargs['cluster_list'].items():
        for cluster in clusters:
            if not cluster['MultiAZ']:
                result = False
                failReason = "MultiAZ is not enabled"
                offenders.append(cluster['DBClusterIdentifier'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }


# checks compliance for RdsInstanceIamAuthenticationEnabled
def rds_instance_iam_authentication_enabled(self, **kwargs) -> dict:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside rds :: rds_instance_iam_authentication_enabled()--- ")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id5.15'
    compliance_type = 'Rds Instance Iam Authentication Enabled'
    description = "Checks if an Amazon Relational Database Service (Amazon RDS) instance has AWS Identity and Access " \
                  "Management (IAM) authentication enabled. The DB Engine should be one of 'mysql', 'postgres', " \
                  "'aurora', 'aurora-mysql', or 'aurora-postgresql'. The DB instance status should be one of " \
                  "'available', 'backing-up', 'storage-optimization', or 'storage-full'"
    resource_type = 'RDS'
    risk_level = 'Medium'

    if 'exception' in kwargs.keys() and kwargs['exception']:
        return {
            'Result': False,
            'failReason': kwargs['exception_text'],
            'resource_type': resource_type,
            'Offenders': offenders,
            'Compliance_type': compliance_type,
            'Description': description,
            'Risk Level': risk_level,
            'ControlId': control_id
        }

    for region, instances in kwargs['rds_lst'].items():
        for instance in instances:
            if instance['Engine'] in ['mysql', 'postgres', 'aurora', 'aurora-mysql', 'aurora-postgresql']:
                if instance['DBInstanceStatus'] in ['available', 'backing-up', 'storage-optimization', 'storage-full']:
                    if not instance['IAMDatabaseAuthenticationEnabled']:
                        result = False
                        failReason = 'IAM Database Authentication is not enabled'
                        offenders.append(instance['DBInstanceIdentifier'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level,
        'ControlId': control_id
    }
